import { __decorate, __metadata } from "tslib";
import { MdcComponent } from '@aurelia2-mdc-web/base';
import { MDCDialogFoundation, strings, util } from '@material/dialog';
import { INode, bindable, children, customElement, inject } from 'aurelia';
import { closest, matches } from '@material/dom/ponyfill';
import template from './mdc-dialog.html';
const LAYOUT_EVENTS = ['resize', 'orientationchange'];
let dialogId = 0;
strings.OPENING_EVENT = strings.OPENING_EVENT.toLowerCase();
strings.CLOSING_EVENT = strings.CLOSING_EVENT.toLowerCase();
strings.OPENED_EVENT = strings.OPENED_EVENT.toLowerCase();
/**
 * @selector mdc-dialog
 */
let MdcDialog = class MdcDialog extends MdcComponent {
    id = ++dialogId;
    contentId = `mdc-dialog-content-${this.id}`;
    titleId = `mdc-dialog-title-${this.id}`;
    buttons; // assigned in initialize()
    mdcFocusTrap;
    content_; // assigned in initialize()
    /** Action returned when the dialog is closed via the scrim click */
    scrimClickAction;
    async scrimClickActionChanged() {
        await this.initialised;
        this.foundation?.setScrimClickAction(this.scrimClickAction);
    }
    /** Action returned when the dialog is closed via the ESC key */
    escapeKeyAction;
    async escapeKeyActionChanged() {
        await this.initialised;
        this.foundation?.setEscapeKeyAction(this.escapeKeyAction);
    }
    delayFocusTrap;
    fullscreen;
    sheet;
    get defaultButton() {
        return this.root.querySelector(`[${strings.BUTTON_DEFAULT_ATTRIBUTE}]`);
    }
    get content() {
        return this.root.querySelector(`${strings.CONTENT_SELECTOR}`);
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    async initialise() {
        this.buttons = [].slice.call(this.root.querySelectorAll(strings.BUTTON_SELECTOR));
        const content = this.root.querySelector('mdc-dialog-content');
        content?.setAttribute('id', this.contentId);
        const title = this.root.querySelector('mdc-dialog-title');
        title?.setAttribute('id', this.titleId);
    }
    createFocusTrap() {
        this.mdcFocusTrap?.create();
    }
    destroy() {
        this.handleClosing_();
        super.destroy();
    }
    handleClick_(evt) {
        this.foundation?.handleClick(evt);
        return true;
    }
    handleKeydown_(evt) {
        this.foundation?.handleKeydown(evt);
        return true;
    }
    handleEvent(evt) {
        switch (evt.currentTarget) {
            case window:
                switch (evt.type) {
                    case 'resize':
                    case 'orientationchange':
                        this.foundation?.layout();
                        break;
                }
                break;
            case document:
                switch (evt.type) {
                    case 'keydown':
                        this.foundation?.handleDocumentKeydown(evt);
                        break;
                }
                break;
        }
    }
    handleOpening_() {
        LAYOUT_EVENTS.forEach((evtType) => window.addEventListener(evtType, this));
        document.addEventListener('keydown', this);
    }
    handleClosing_() {
        LAYOUT_EVENTS.forEach((evtType) => window.removeEventListener(evtType, this));
        document.removeEventListener('keydown', this);
    }
    /** Opens the dialog */
    open() {
        this.foundation?.open();
    }
    /**
     * Closes the dialog
     * @param action Action to close the dialog with
     */
    close(action = '') {
        this.foundation?.close(action);
    }
    getDefaultFoundation() {
        // DO NOT INLINE this variable. For backward compatibility, foundations take a Partial<MDCFooAdapter>.
        // To ensure we don't accidentally omit any methods, we need a separate, strongly typed adapter variable.
        const adapter = {
            addBodyClass: (className) => document.body.classList.add(className),
            addClass: (className) => this.root.classList.add(className),
            areButtonsStacked: () => util.areTopsMisaligned(this.buttons),
            clickDefaultButton: () => {
                if (this.defaultButton) {
                    this.defaultButton.click();
                }
            },
            eventTargetMatches: (target, selector) => target ? matches(target, selector) : false,
            getActionFromEvent: (evt) => {
                if (!evt.target) {
                    return '';
                }
                const element = closest(evt.target, `[${strings.ACTION_ATTRIBUTE}]`);
                return element?.getAttribute(strings.ACTION_ATTRIBUTE) ?? null;
            },
            getInitialFocusEl: () => this.getInitialFocusEl(),
            hasClass: (className) => this.root.classList.contains(className),
            isContentScrollable: () => util.isScrollable(this.content),
            notifyClosed: (action) => this.emit(strings.CLOSED_EVENT, action ? { action } : {}),
            notifyClosing: (action) => this.emit(strings.CLOSING_EVENT, action ? { action } : {}),
            notifyOpened: () => this.emit(strings.OPENED_EVENT, {}),
            notifyOpening: () => this.emit(strings.OPENING_EVENT, {}),
            releaseFocus: () => {
                this.mdcFocusTrap?.releaseFocus();
            },
            removeBodyClass: (className) => document.body.classList.remove(className),
            removeClass: (className) => this.root.classList.remove(className),
            reverseButtons: () => {
                this.buttons.reverse();
                this.buttons.forEach((button) => {
                    button.parentElement.appendChild(button);
                });
            },
            trapFocus: () => {
                this.mdcFocusTrap?.trapFocus();
            },
            registerContentEventHandler: (evt, handler) => {
                if (this.content instanceof HTMLElement) {
                    this.content.addEventListener(evt, handler);
                }
            },
            deregisterContentEventHandler: (evt, handler) => {
                if (this.content instanceof HTMLElement) {
                    this.content.removeEventListener(evt, handler);
                }
            },
            isScrollableContentAtTop: () => {
                return util.isScrollAtTop(this.content);
            },
            isScrollableContentAtBottom: () => {
                return util.isScrollAtBottom(this.content);
            },
            registerWindowEventHandler: (evt, handler) => {
                window.addEventListener(evt, handler);
            },
            deregisterWindowEventHandler: (evt, handler) => {
                window.removeEventListener(evt, handler);
            },
        };
        return new MDCDialogFoundation(adapter);
    }
    getInitialFocusEl = () => {
        return this.root.querySelector(`[${strings.INITIAL_FOCUS_ATTRIBUTE}] input, [${strings.INITIAL_FOCUS_ATTRIBUTE}] .mdc-select__anchor`);
    };
};
__decorate([
    children('mdc-dialog-content'),
    __metadata("design:type", Array)
], MdcDialog.prototype, "content_", void 0);
__decorate([
    bindable,
    __metadata("design:type", String)
], MdcDialog.prototype, "scrimClickAction", void 0);
__decorate([
    bindable,
    __metadata("design:type", String)
], MdcDialog.prototype, "escapeKeyAction", void 0);
__decorate([
    bindable,
    __metadata("design:type", Boolean)
], MdcDialog.prototype, "delayFocusTrap", void 0);
__decorate([
    bindable,
    __metadata("design:type", Boolean)
], MdcDialog.prototype, "fullscreen", void 0);
__decorate([
    bindable,
    __metadata("design:type", Boolean)
], MdcDialog.prototype, "sheet", void 0);
MdcDialog = __decorate([
    inject(INode),
    customElement({
        name: 'mdc-dialog',
        template,
    })
], MdcDialog);
export { MdcDialog };
//# sourceMappingURL=mdc-dialog.js.map